@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - AVDTP parser
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
  *
@@ -17,16 +17,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: avdtp.c,v 1.12 2005/02/01 14:36:57 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
 #include <errno.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <sys/types.h>
@@ -143,9 +145,8 @@ static char *codec2str(uint8_t type, uint8_t codec)
 			return "Reserved";
 		}
 		break;
-	default:
-		return "Unknown";
 	}
+	return "Unknown";
 }
 
 static char *cat2str(uint8_t cat)
