@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - CAPI parser
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
  *
@@ -17,17 +17,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: capi.c,v 1.7 2005/02/01 14:36:59 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include <stdlib.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <sys/types.h>
@@ -764,7 +766,7 @@ static void cmd_manufacturer(int level, uint8_t subcmd, struct frame *frm)
 	frm->ptr += 4;
 	frm->len -= 4;
 
-	if (!strncmp(id, "AVM!", 4)) {
+	if (!strncmp((char *) id, "AVM!", 4)) {
 		class = CAPI_U32(frm);
 		func = CAPI_U32(frm);
 		len = CAPI_U8(frm);
