@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - HCI parser
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
  *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
@@ -18,17 +18,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: hci.c,v 1.50 2005/06/17 11:51:24 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include <stdlib.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <sys/types.h>
@@ -48,7 +50,8 @@ static inline uint16_t get_manufacturer(void)
 	return (manufacturer == DEFAULT_COMPID ? parser.defcompid : manufacturer);
 }
 
-static char *event_str[] = {
+#define EVENT_NUM 47
+static char *event_str[EVENT_NUM + 1] = {
 	"Unknown",
 	"Inquiry Complete",
 	"Inquiry Result",
@@ -94,11 +97,13 @@ static char *event_str[] = {
 	"Unknown",
 	"Unknown",
 	"Synchronous Connect Complete",
-	"Synchronous Connect Changed"
+	"Synchronous Connect Changed",
+	"Sniff Subrate",
+	"Extended Inquiry Result",
 };
-#define EVENT_NUM 43
 
-static char *cmd_linkctl_str[] = {
+#define CMD_LINKCTL_NUM 41
+static char *cmd_linkctl_str[CMD_LINKCTL_NUM + 1] = {
 	"Unknown",
 	"Inquiry",
 	"Inquiry Cancel",
@@ -141,11 +146,11 @@ static char *cmd_linkctl_str[] = {
 	"Unknown",
 	"Setup Synchronous Connection",
 	"Accept Synchronous Connection",
-	"Reject Synchronous Connection"
+	"Reject Synchronous Connection",
 };
-#define CMD_LINKCTL_NUM 42
 
-static char *cmd_linkpol_str[] = {
+#define CMD_LINKPOL_NUM 17
+static char *cmd_linkpol_str[CMD_LINKPOL_NUM + 1] = {
 	"Unknown",
 	"Hold Mode",
 	"Unknown",
@@ -162,11 +167,12 @@ static char *cmd_linkpol_str[] = {
 	"Write Link Policy Settings",
 	"Read Default Link Policy Settings",
 	"Write Default Link Policy Settings",
-	"Flow Specification"
+	"Flow Specification",
+	"Sniff Subrate",
 };
-#define CMD_LINKPOL_NUM 16
 
-static char *cmd_hostctl_str[] = {
+#define CMD_HOSTCTL_NUM 82
+static char *cmd_hostctl_str[CMD_HOSTCTL_NUM + 1] = {
 	"Unknown",
 	"Set Event Mask",
 	"Unknown",
@@ -240,11 +246,20 @@ static char *cmd_hostctl_str[] = {
 	"Read Page Scan Type",
 	"Write Page Scan Type",
 	"Read AFH Channel Assessment Mode",
-	"Write AFH Channel Assessment Mode"
+	"Write AFH Channel Assessment Mode",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Read Extended Inquiry Response",
+	"Write Extended Inquiry Response",
 };
-#define CMD_HOSTCTL_NUM 73
 
-static char *cmd_info_str[] = {
+#define CMD_INFO_NUM 9
+static char *cmd_info_str[CMD_INFO_NUM + 1] = {
 	"Unknown",
 	"Read Local Version Information",
 	"Read Local Supported Commands",
@@ -254,11 +269,11 @@ static char *cmd_info_str[] = {
 	"Unknown",
 	"Read Country Code",
 	"Unknown",
-	"Read BD ADDR"
+	"Read BD ADDR",
 };
-#define CMD_INFO_NUM 9
 
-static char *cmd_status_str[] = {
+#define CMD_STATUS_NUM 7
+static char *cmd_status_str[CMD_STATUS_NUM + 1] = {
 	"Unknown",
 	"Read Failed Contact Counter",
 	"Reset Failed Contact Counter",
@@ -266,11 +281,11 @@ static char *cmd_status_str[] = {
 	"Unknown",
 	"Read RSSI",
 	"Read AFH Channel Map",
-	"Read Clock"
+	"Read Clock",
 };
-#define CMD_STATUS_NUM 7
 
-static char *error_code_str[] = {
+#define ERROR_CODE_NUM 53
+static char *error_code_str[ERROR_CODE_NUM + 1] = {
 	"Success",
 	"Unknown HCI Command",
 	"Unknown Connection Identifier",
@@ -324,9 +339,8 @@ static char *error_code_str[] = {
 	"Role Switch Pending",
 	"Reserved",
 	"Reserved Slot Violation",
-	"Role Switch Failed"
+	"Role Switch Failed",
 };
-#define ERROR_CODE_NUM 53
 
 static char *status2str(uint8_t status)
 {
@@ -442,6 +456,53 @@ static char *airmode2str(uint8_t mode)
 	}
 }
 
+static inline void ext_inquiry_response_dump(int level, struct frame *frm)
+{
+	void *ptr = frm->ptr;
+	uint32_t len = frm->len;
+	uint8_t type, length;
+	char *str;
+	int i;
+
+	length = get_u8(frm);
+
+	while (length > 0) {
+		type = get_u8(frm);
+		length--;
+
+		switch (type) {
+		case 0x08:
+		case 0x09:
+			str = malloc(length + 1);
+			if (str) {
+				snprintf(str, length + 1, "%s", (char *) frm->ptr);
+				for (i = 0; i < length; i++)
+				if (!isprint(str[i]))
+					str[i] = '.';
+				p_indent(level, frm);
+				printf("%s local name: \'%s\'\n",
+					type == 0x08 ? "Shortened" : "Complete", str);
+				free(str);
+			}
+			break;
+
+		default:
+			p_indent(level, frm);
+			printf("Unknown type 0x%02x with %d bytes data\n",
+								type, length);
+			break;
+		}
+
+		frm->ptr += length;
+		frm->len -= length;
+
+		length = get_u8(frm);
+	}
+
+	frm->ptr = ptr + (EXTENDED_INQUIRY_INFO_SIZE - INQUIRY_INFO_WITH_RSSI_SIZE);
+	frm->len = len + (EXTENDED_INQUIRY_INFO_SIZE - INQUIRY_INFO_WITH_RSSI_SIZE);
+}
+
 static inline void generic_command_dump(int level, struct frame *frm)
 {
 	uint16_t handle = btohs(htons(get_u16(frm)));
@@ -462,7 +523,7 @@ static inline void bdaddr_command_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	ba2str(bdaddr, addr);
-        printf("bdaddr %s\n", addr);
+	printf("bdaddr %s\n", addr);
 
 	raw_dump(level, frm);
 }
@@ -569,7 +630,10 @@ static inline void pin_code_reply_dump(int level, struct frame *frm)
 	p_indent(level, frm);
 	ba2str(&cp->bdaddr, addr);
 	memset(pin, 0, sizeof(pin));
-	memcpy(pin, cp->pin_code, cp->pin_len);
+	if (parser.flags & DUMP_NOVENDOR)
+		memset(pin, '*', cp->pin_len);
+	else
+		memcpy(pin, cp->pin_code, cp->pin_len);
 	printf("bdaddr %s len %d pin \'%s\'\n", addr, cp->pin_len, pin);
 }
 
@@ -583,7 +647,10 @@ static inline void link_key_reply_dump(int level, struct frame *frm)
 	ba2str(&cp->bdaddr, addr);
 	printf("bdaddr %s key ", addr);
 	for (i = 0; i < 16; i++)
-		printf("%2.2X", cp->link_key[i]);
+		if (parser.flags & DUMP_NOVENDOR)
+			printf("**");
+		else
+			printf("%2.2X", cp->link_key[i]);
 	printf("\n");
 }
 
@@ -630,8 +697,27 @@ static inline void read_remote_ext_features_dump(int level, struct frame *frm)
 {
 	read_remote_ext_features_cp *cp = frm->ptr;
 
-        p_indent(level, frm);
-        printf("handle %d page %d\n", btohs(cp->handle), cp->page_num);
+	p_indent(level, frm);
+	printf("handle %d page %d\n", btohs(cp->handle), cp->page_num);
+}
+
+static inline void hold_mode_dump(int level, struct frame *frm)
+{
+	hold_mode_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d max %d min %d\n", btohs(cp->handle),
+			btohs(cp->max_interval), btohs(cp->min_interval));
+}
+
+static inline void sniff_mode_dump(int level, struct frame *frm)
+{
+	sniff_mode_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d max %d min %d attempt %d timeout %d\n",
+		btohs(cp->handle), btohs(cp->max_interval),
+		btohs(cp->min_interval), btohs(cp->attempt), btohs(cp->timeout));
 }
 
 static inline void write_link_policy_dump(int level, struct frame *frm)
@@ -651,6 +737,22 @@ static inline void write_link_policy_dump(int level, struct frame *frm)
 	}
 }
 
+static inline void sniff_subrate_dump(int level, struct frame *frm)
+{
+	sniff_subrate_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d\n", btohs(cp->handle));
+
+	p_indent(level, frm);
+	printf("max latency remote %d local %d\n",
+		btohs(cp->max_remote_latency), btohs(cp->max_local_latency));
+
+	p_indent(level, frm);
+	printf("min timeout remote %d local %d\n",
+		btohs(cp->min_remote_timeout), btohs(cp->min_local_timeout));
+}
+
 static inline void set_event_mask_dump(int level, struct frame *frm)
 {
 	set_event_mask_cp *cp = frm->ptr;
@@ -666,26 +768,43 @@ static inline void set_event_mask_dump(int level, struct frame *frm)
 static inline void set_event_flt_dump(int level, struct frame *frm)
 {
 	set_event_flt_cp *cp = frm->ptr;
+	uint8_t dev_class[3], dev_mask[3];
+	char addr[18];
 
 	p_indent(level, frm);
-	printf("type %d condition %d\n", cp->flt_type, cp->cond_type);
+	printf("type %d condition %d\n", cp->flt_type,
+				(cp->flt_type == 0) ? 0 : cp->cond_type);
 
 	switch (cp->flt_type) {
 	case FLT_CLEAR_ALL:
+		p_indent(level, frm);
 		printf("Clear all filters\n");
 		break;
 	case FLT_INQ_RESULT:
+		p_indent(level, frm);
 		printf("Inquiry result");
 		switch (cp->cond_type) {
 		case INQ_RESULT_RETURN_ALL:
+			printf(" for all devices\n");
+			break;
 		case INQ_RESULT_RETURN_CLASS:
+			memcpy(dev_class, cp->condition, 3);
+			memcpy(dev_mask, cp->condition + 3, 3);
+			printf(" with class 0x%2.2x%2.2x%2.2x mask 0x%2.2x%2.2x%2.2x\n",
+				dev_class[2], dev_class[1], dev_class[0],
+				dev_mask[2], dev_mask[1], dev_mask[0]);
+			break;
 		case INQ_RESULT_RETURN_BDADDR:
+			ba2str((bdaddr_t *) cp->condition, addr);
+			printf(" with bdaddr %s\n", addr);
+			break;
 		default:
 			printf("\n");
 			break;
 		}
 		break;
 	case FLT_CONN_SETUP:
+		p_indent(level, frm);
 		printf("Connection setup");
 		switch (cp->cond_type) {
 		case CONN_SETUP_ALLOW_ALL:
@@ -731,7 +850,10 @@ static inline void return_link_keys_dump(int level, struct frame *frm)
 		p_indent(level, frm);
 		printf("bdaddr %s key ", addr);
 		for (i = 0; i < 16; i++)
-			printf("%2.2X", key[i]);
+			if (parser.flags & DUMP_NOVENDOR)
+				printf("**");
+			else
+				printf("%2.2X", key[i]);
 		printf("\n");
 
 		frm->ptr += 2;
@@ -762,7 +884,7 @@ static inline void write_class_of_dev_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("class 0x%2.2x%2.2x%2.2x\n",
-		cp->dev_class[2], cp->dev_class[1], cp->dev_class[0]);
+			cp->dev_class[2], cp->dev_class[1], cp->dev_class[0]);
 }
 
 static inline void write_voice_setting_dump(int level, struct frame *frm)
@@ -841,7 +963,20 @@ static inline void write_link_supervision_timeout_dump(int level, struct frame *
 
 	p_indent(level, frm);
 	printf("handle %d timeout %d\n",
-		btohs(cp->handle), btohs(cp->link_sup_to));
+				btohs(cp->handle), btohs(cp->link_sup_to));
+}
+
+static inline void write_ext_inquiry_response_dump(int level, struct frame *frm)
+{
+	write_ext_inquiry_response_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("fec 0x%2.2x\n", cp->fec);
+
+	frm->ptr++;
+	frm->len--;
+
+	ext_inquiry_response_dump(level, frm);
 }
 
 static inline void request_transmit_power_level_dump(int level, struct frame *frm)
@@ -850,8 +985,8 @@ static inline void request_transmit_power_level_dump(int level, struct frame *fr
 
 	p_indent(level, frm);
 	printf("handle %d type %d (%s)\n",
-		btohs(cp->handle), cp->type,
-		cp->type ? "maximum" : "current");
+					btohs(cp->handle), cp->type,
+					cp->type ? "maximum" : "current");
 }
 
 static inline void request_local_ext_features_dump(int level, struct frame *frm)
@@ -868,8 +1003,8 @@ static inline void request_clock_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("handle %d which %d (%s)\n",
-		btohs(cp->handle), cp->which_clock,
-		cp->which_clock ? "piconet" : "local");
+					btohs(cp->handle), cp->which_clock,
+					cp->which_clock ? "piconet" : "local");
 }
 
 static inline void command_dump(int level, struct frame *frm)
@@ -882,17 +1017,21 @@ static inline void command_dump(int level, struct frame *frm)
 	if (p_filter(FILT_HCI))
 		return;
 
-	p_indent(level, frm);
+	if (ogf == OGF_VENDOR_CMD && (parser.flags & DUMP_NOVENDOR))
+		return;
 
+	p_indent(level, frm);
 	printf("HCI Command: %s (0x%2.2x|0x%4.4x) plen %d\n", 
-		opcode2str(opcode), ogf, ocf, hdr->plen);
+				opcode2str(opcode), ogf, ocf, hdr->plen);
 
 	frm->ptr += HCI_COMMAND_HDR_SIZE;
 	frm->len -= HCI_COMMAND_HDR_SIZE;
 
-	if (ogf == OGF_VENDOR_CMD && ocf == 0 && get_manufacturer() == 10) {
-		csr_dump(level + 1, frm);
-		return;
+	if (ogf == OGF_VENDOR_CMD) {
+	       	if (ocf == 0 && get_manufacturer() == 10) {
+			csr_dump(level + 1, frm);
+			return;
+		}
 	}
 
 	if (!(parser.flags & DUMP_VERBOSE)) {
@@ -970,6 +1109,13 @@ static inline void command_dump(int level, struct frame *frm)
 
 	case OGF_LINK_POLICY:
 		switch (ocf) {
+		case OCF_HOLD_MODE:
+		case OCF_PARK_MODE:
+			hold_mode_dump(level + 1, frm);
+			return;
+		case OCF_SNIFF_MODE:
+			sniff_mode_dump(level + 1, frm);
+			return;
 		case OCF_EXIT_SNIFF_MODE:
 		case OCF_EXIT_PARK_MODE:
 		case OCF_ROLE_DISCOVERY:
@@ -982,6 +1128,9 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_WRITE_LINK_POLICY:
 			write_link_policy_dump(level + 1, frm);
 			return;
+		case OCF_SNIFF_SUBRATE:
+			sniff_subrate_dump(level + 1, frm);
+			return;
 		}
 		break;
 
@@ -1048,6 +1197,9 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_WRITE_LINK_SUPERVISION_TIMEOUT:
 			write_link_supervision_timeout_dump(level + 1, frm);
 			return;
+		case OCF_WRITE_EXT_INQUIRY_RESPONSE:
+			write_ext_inquiry_response_dump(level + 1, frm);
+			return;
 		}
 		break;
 
@@ -1157,7 +1309,7 @@ static inline void read_stored_link_key_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x max %d num %d\n",
-		rp->status, rp->max_keys, rp->num_keys);
+				rp->status, rp->max_keys, rp->num_keys);
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1219,7 +1371,7 @@ static inline void read_class_of_dev_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x class 0x%2.2x%2.2x%2.2x\n", rp->status,
-		rp->dev_class[2], rp->dev_class[1], rp->dev_class[0]);
+			rp->dev_class[2], rp->dev_class[1], rp->dev_class[0]);
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1233,7 +1385,7 @@ static inline void read_voice_setting_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x voice setting 0x%4.4x\n",
-		rp->status, btohs(rp->voice_setting));
+					rp->status, btohs(rp->voice_setting));
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1296,7 +1448,7 @@ static inline void read_page_activity_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x interval %d window %d\n",
-		rp->status, btohs(rp->interval), btohs(rp->window));
+			rp->status, btohs(rp->interval), btohs(rp->window));
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1336,7 +1488,7 @@ static inline void read_link_supervision_timeout_dump(int level, struct frame *f
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d timeout %d\n",
-		rp->status, btohs(rp->handle), btohs(rp->link_sup_to));
+			rp->status, btohs(rp->handle), btohs(rp->link_sup_to));
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1350,7 +1502,7 @@ static inline void read_transmit_power_level_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d level %d\n",
-		rp->status, btohs(rp->handle), rp->level);
+				rp->status, btohs(rp->handle), rp->level);
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1358,6 +1510,24 @@ static inline void read_transmit_power_level_dump(int level, struct frame *frm)
 	}
 }
 
+static inline void read_ext_inquiry_response_dump(int level, struct frame *frm)
+{
+	read_ext_inquiry_response_rp *rp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x fec 0x%2.2x\n", rp->status, rp->fec);
+
+	if (rp->status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(rp->status));
+	} else {
+		frm->ptr += 2;
+		frm->len -= 2;
+
+		ext_inquiry_response_dump(level, frm);
+	}
+}
+
 static inline void read_local_version_dump(int level, struct frame *frm)
 {
 	read_local_version_rp *rp = frm->ptr;
@@ -1372,15 +1542,15 @@ static inline void read_local_version_dump(int level, struct frame *frm)
 	} else {
 		p_indent(level, frm);
 		printf("HCI Version: %s (0x%x) HCI Revision: 0x%x\n",
-			hci_vertostr(rp->hci_ver), rp->hci_ver,
-			btohs(rp->hci_rev));
+					hci_vertostr(rp->hci_ver),
+					rp->hci_ver, btohs(rp->hci_rev));
 		p_indent(level, frm);
 		printf("LMP Version: %s (0x%x) LMP Subversion: 0x%x\n",
-			lmp_vertostr(rp->lmp_ver), rp->lmp_ver,
-			btohs(rp->lmp_subver));
+					lmp_vertostr(rp->lmp_ver),
+					rp->lmp_ver, btohs(rp->lmp_subver));
 		p_indent(level, frm);
 		printf("Manufacturer: %s (%d)\n",
-			bt_compidtostr(manufacturer), manufacturer);
+				bt_compidtostr(manufacturer), manufacturer);
 	}
 }
 
@@ -1468,8 +1638,8 @@ static inline void read_buffer_size_dump(int level, struct frame *frm)
 	} else {
 		p_indent(level, frm);
 		printf("ACL MTU %d:%d SCO MTU %d:%d\n",
-			btohs(rp->acl_mtu), btohs(rp->acl_max_pkt),
-			rp->sco_mtu, btohs(rp->sco_max_pkt));
+				btohs(rp->acl_mtu), btohs(rp->acl_max_pkt),
+				rp->sco_mtu, btohs(rp->sco_max_pkt));
 	}
 }
 
@@ -1479,7 +1649,7 @@ static inline void read_link_quality_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d lq %d\n",
-		rp->status, btohs(rp->handle), rp->link_quality);
+			rp->status, btohs(rp->handle), rp->link_quality);
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1493,7 +1663,7 @@ static inline void read_rssi_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d rssi %d\n",
-		rp->status, btohs(rp->handle), rp->rssi);
+				rp->status, btohs(rp->handle), rp->rssi);
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1508,7 +1678,7 @@ static inline void read_afh_map_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d mode %d\n",
-		rp->status, btohs(rp->handle), rp->mode);
+				rp->status, btohs(rp->handle), rp->mode);
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1528,8 +1698,8 @@ static inline void read_clock_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d clock 0x%4.4x accuracy %d\n",
-		rp->status, btohs(rp->handle),
-		btohl(rp->clock), btohs(rp->accuracy));
+					rp->status, btohs(rp->handle),
+					btohl(rp->clock), btohs(rp->accuracy));
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1544,9 +1714,12 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 	uint16_t ogf = cmd_opcode_ogf(opcode);
 	uint16_t ocf = cmd_opcode_ocf(opcode);
 
+	if (ogf == OGF_VENDOR_CMD && (parser.flags & DUMP_NOVENDOR))
+		return;
+
 	p_indent(level, frm);
 	printf("%s (0x%2.2x|0x%4.4x) ncmd %d\n",
-		opcode2str(opcode), ogf, ocf, evt->ncmd);
+				opcode2str(opcode), ogf, ocf, evt->ncmd);
 
 	frm->ptr += EVT_CMD_COMPLETE_SIZE;
 	frm->len -= EVT_CMD_COMPLETE_SIZE;
@@ -1578,6 +1751,7 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 	case OGF_LINK_POLICY:
 		switch (ocf) {
 		case OCF_WRITE_LINK_POLICY:
+		case OCF_SNIFF_SUBRATE:
 			generic_response_dump(level, frm);
 			return;
 		}
@@ -1635,6 +1809,9 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 		case OCF_READ_TRANSMIT_POWER_LEVEL:
 			read_transmit_power_level_dump(level, frm);
 			return;
+		case OCF_READ_EXT_INQUIRY_RESPONSE:
+			read_ext_inquiry_response_dump(level, frm);
+			return;
 		case OCF_FLUSH:
 		case OCF_WRITE_LINK_SUPERVISION_TIMEOUT:
 			generic_response_dump(level, frm);
@@ -1659,6 +1836,7 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 		case OCF_WRITE_INQUIRY_MODE:
 		case OCF_WRITE_AFH_MODE:
 		case OCF_SET_AFH_CLASSIFICATION:
+		case OCF_WRITE_EXT_INQUIRY_RESPONSE:
 			status_response_dump(level, frm);
 			return;
 		}
@@ -1716,12 +1894,15 @@ static inline void cmd_status_dump(int level, struct frame *frm)
 {
 	evt_cmd_status *evt = frm->ptr;
 	uint16_t opcode = btohs(evt->opcode);
+	uint16_t ogf = cmd_opcode_ogf(opcode);
+	uint16_t ocf = cmd_opcode_ocf(opcode);
+
+	if (ogf == OGF_VENDOR_CMD && (parser.flags & DUMP_NOVENDOR))
+		return;
 
 	p_indent(level, frm);
 	printf("%s (0x%2.2x|0x%4.4x) status 0x%2.2x ncmd %d\n",
-		opcode2str(opcode),
-		cmd_opcode_ogf(opcode), cmd_opcode_ocf(opcode),
-		evt->status, evt->ncmd);
+			opcode2str(opcode), ogf, ocf, evt->status, evt->ncmd);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -1767,8 +1948,8 @@ static inline void conn_complete_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d bdaddr %s type %s encrypt 0x%2.2x\n",
-		evt->status, btohs(evt->handle), addr,
-		evt->link_type == 1 ? "ACL" : "SCO", evt->encr_mode);
+			evt->status, btohs(evt->handle), addr,
+			evt->link_type == 1 ? "ACL" : "SCO", evt->encr_mode);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -1785,8 +1966,8 @@ static inline void conn_request_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("bdaddr %s class 0x%2.2x%2.2x%2.2x type %s\n",
-		addr, evt->dev_class[2], evt->dev_class[1],
-		evt->dev_class[0], evt->link_type == 1 ? "ACL" : "SCO");
+			addr, evt->dev_class[2], evt->dev_class[1],
+			evt->dev_class[0], evt->link_type == 1 ? "ACL" : "SCO");
 }
 
 static inline void disconn_complete_dump(int level, struct frame *frm)
@@ -1795,7 +1976,7 @@ static inline void disconn_complete_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d reason 0x%2.2x\n",
-		evt->status, btohs(evt->handle), evt->reason);
+				evt->status, btohs(evt->handle), evt->reason);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -1836,7 +2017,7 @@ static inline void master_link_key_complete_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d flag %d\n",
-		evt->status, btohs(evt->handle), evt->key_flag);
+				evt->status, btohs(evt->handle), evt->key_flag);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -1850,7 +2031,7 @@ static inline void encrypt_change_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d encrypt 0x%2.2x\n",
-		evt->status, btohs(evt->handle), evt->encrypt);
+				evt->status, btohs(evt->handle), evt->encrypt);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -1906,7 +2087,7 @@ static inline void qos_setup_complete_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d flags %d\n",
-		evt->status, btohs(evt->handle), evt->flags);
+				evt->status, btohs(evt->handle), evt->flags);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -1933,7 +2114,7 @@ static inline void role_change_dump(int level, struct frame *frm)
 	p_indent(level, frm);
 	ba2str(&evt->bdaddr, addr);
 	printf("status 0x%2.2x bdaddr %s role 0x%2.2x\n",
-		evt->status, addr, evt->role);
+						evt->status, addr, evt->role);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -1996,7 +2177,10 @@ static inline void link_key_notify_dump(int level, struct frame *frm)
 	ba2str(&evt->bdaddr, addr);
 	printf("bdaddr %s key ", addr);
 	for (i = 0; i < 16; i++)
-		printf("%2.2X", evt->link_key[i]);
+		if (parser.flags & DUMP_NOVENDOR)
+			printf("**");
+		else
+			printf("%2.2X", evt->link_key[i]);
 	printf(" type %d\n", evt->key_type);
 }
 
@@ -2038,7 +2222,7 @@ static inline void conn_ptype_changed_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d ptype 0x%4.4x\n",
-		evt->status, btohs(evt->handle), ptype);
+				evt->status, btohs(evt->handle), ptype);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -2069,8 +2253,8 @@ static inline void flow_spec_complete_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d flags %d %s\n",
-		evt->status, btohs(evt->handle), evt->flags,
-		evt->direction == 0 ? "outgoing" : "incoming");
+				evt->status, btohs(evt->handle), evt->flags,
+				evt->direction == 0 ? "outgoing" : "incoming");
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -2136,8 +2320,8 @@ static inline void read_remote_ext_features_complete_dump(int level, struct fram
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d page %d max %d\n",
-		evt->status, btohs(evt->handle),
-		evt->page_num, evt->max_page_num);
+					evt->status, btohs(evt->handle),
+					evt->page_num, evt->max_page_num);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -2160,8 +2344,8 @@ static inline void sync_conn_complete_dump(int level, struct frame *frm)
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d bdaddr %s type %s\n",
-		evt->status, btohs(evt->handle), addr,
-		evt->link_type == 0 ? "SCO" : "eSCO");
+					evt->status, btohs(evt->handle), addr,
+					evt->link_type == 0 ? "SCO" : "eSCO");
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -2177,8 +2361,7 @@ static inline void sync_conn_changed_dump(int level, struct frame *frm)
 	evt_sync_conn_changed *evt = frm->ptr;
 
 	p_indent(level, frm);
-	printf("status 0x%2.2x handle %d\n",
-		evt->status, btohs(evt->handle));
+	printf("status 0x%2.2x handle %d\n", evt->status, btohs(evt->handle));
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -2186,6 +2369,52 @@ static inline void sync_conn_changed_dump(int level, struct frame *frm)
 	}
 }
 
+static inline void sniff_subrate_event_dump(int level, struct frame *frm)
+{
+	evt_sniff_subrate *evt = frm->ptr;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x handle %d\n", evt->status, btohs(evt->handle));
+
+	if (evt->status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(evt->status));
+	} else {
+		p_indent(level, frm);
+		printf("max latency remote %d local %d\n",
+					btohs(evt->max_remote_latency),
+					btohs(evt->max_local_latency));
+
+		p_indent(level, frm);
+		printf("min timeout remote %d local %d\n",
+					btohs(evt->min_remote_timeout),
+					btohs(evt->min_local_timeout));
+	}
+}
+
+static inline void extended_inq_result_dump(int level, struct frame *frm)
+{
+	uint8_t num = get_u8(frm);
+	char addr[18];
+	int i;
+
+	for (i = 0; i < num; i++) {
+		extended_inquiry_info *info = frm->ptr;
+
+		ba2str(&info->bdaddr, addr);
+
+		p_indent(level, frm);
+		printf("bdaddr %s mode %d clkoffset 0x%4.4x class 0x%2.2x%2.2x%2.2x rssi %d\n",
+			addr, info->pscan_rep_mode, btohs(info->clock_offset),
+			info->dev_class[2], info->dev_class[1], info->dev_class[0], info->rssi);
+
+		frm->ptr += INQUIRY_INFO_WITH_RSSI_SIZE;
+		frm->len -= INQUIRY_INFO_WITH_RSSI_SIZE;
+
+		ext_inquiry_response_dump(level, frm);
+	}
+}
+
 static inline void event_dump(int level, struct frame *frm)
 {
 	hci_event_hdr *hdr = frm->ptr;
@@ -2194,23 +2423,30 @@ static inline void event_dump(int level, struct frame *frm)
 	if (p_filter(FILT_HCI))
 		return;
 
-	p_indent(level, frm);
-
 	if (event <= EVENT_NUM) {
+		p_indent(level, frm);
 		printf("HCI Event: %s (0x%2.2x) plen %d\n",
-			event_str[hdr->evt], hdr->evt, hdr->plen);
+					event_str[hdr->evt], hdr->evt, hdr->plen);
 	} else if (hdr->evt == EVT_TESTING) {
+		p_indent(level, frm);
 		printf("HCI Event: Testing (0x%2.2x) plen %d\n", hdr->evt, hdr->plen);
 	} else if (hdr->evt == EVT_VENDOR) {
+		if (parser.flags & DUMP_NOVENDOR)
+			return;
+
+		p_indent(level, frm);
 		printf("HCI Event: Vendor (0x%2.2x) plen %d\n", hdr->evt, hdr->plen);
+
 		if (get_manufacturer() == 10) {
 			frm->ptr += HCI_EVENT_HDR_SIZE;
 			frm->len -= HCI_EVENT_HDR_SIZE;
 			csr_dump(level + 1, frm);
 			return;
 		}
-	} else
+	} else {
+		p_indent(level, frm);
 		printf("HCI Event: code 0x%2.2x plen %d\n", hdr->evt, hdr->plen);
+	}
 
 	frm->ptr += HCI_EVENT_HDR_SIZE;
 	frm->len -= HCI_EVENT_HDR_SIZE;
@@ -2336,6 +2572,12 @@ static inline void event_dump(int level, struct frame *frm)
 	case EVT_SYNC_CONN_CHANGED:
 		sync_conn_changed_dump(level + 1, frm);
 		break;
+	case EVT_SNIFF_SUBRATE:
+		sniff_subrate_event_dump(level + 1, frm);
+		break;
+	case EVT_EXTENDED_INQUIRY_RESULT:
+		extended_inq_result_dump(level + 1, frm);
+		break;
 	default:
 		raw_dump(level, frm);
 		break;
@@ -2372,10 +2614,13 @@ static inline void sco_dump(int level, struct frame *frm)
 	hci_sco_hdr *hdr = (void *) frm->ptr;
 	uint16_t handle = btohs(hdr->handle);
 
+	if (frm->audio_fd > 2)
+		write(frm->audio_fd, frm->ptr + HCI_SCO_HDR_SIZE, hdr->dlen);
+
 	if (!p_filter(FILT_SCO)) {
 		p_indent(level, frm);
 		printf("SCO data: handle %d dlen %d\n",
-			acl_handle(handle), hdr->dlen);
+					acl_handle(handle), hdr->dlen);
 		level++;
 
 		frm->ptr += HCI_SCO_HDR_SIZE;
@@ -2389,6 +2634,24 @@ static inline void vendor_dump(int level, struct frame *frm)
 	if (p_filter(FILT_HCI))
 		return;
 
+	if (frm->dev_id == HCI_DEV_NONE) {
+		uint16_t device = btohs(htons(get_u16(frm)));
+		uint16_t proto = btohs(htons(get_u16(frm)));
+		uint16_t type = btohs(htons(get_u16(frm)));
+		uint16_t plen = btohs(htons(get_u16(frm)));
+
+		p_indent(level, frm);
+
+		printf("System %s: device hci%d proto 0x%2.2x type 0x%2.2x plen %d\n",
+			frm->in ? "event" : "command", device, proto, type, plen);
+
+		raw_dump(level, frm);
+		return;
+	}
+
+	if (parser.flags & DUMP_NOVENDOR)
+		return;
+
 	if (get_manufacturer() == 12) {
 		bpa_dump(level, frm);
 		return;
