@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - L2CAP parser
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
  *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
@@ -18,16 +18,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: l2cap.c,v 1.45 2005/05/15 20:28:18 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
-#include <stdlib.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <sys/types.h>
@@ -860,15 +862,17 @@ void l2cap_dump(int level, struct frame *frm)
 		}
 		memcpy(fr->data, frm->ptr, frm->len);
 		fr->data_len = dlen + L2CAP_HDR_SIZE;
-		fr->len = frm->len;
-		fr->ptr = fr->data;
-		fr->in  = frm->in;
-		fr->ts  = frm->ts;
-		fr->handle  = frm->handle;
-		fr->cid     = frm->cid;
-		fr->num     = frm->num;
-		fr->dlci    = frm->dlci;
-		fr->channel = frm->channel;
+		fr->len      = frm->len;
+		fr->ptr      = fr->data;
+		fr->dev_id   = frm->dev_id;
+		fr->in       = frm->in;
+		fr->ts       = frm->ts;
+		fr->handle   = frm->handle;
+		fr->cid      = frm->cid;
+		fr->num      = frm->num;
+		fr->dlci     = frm->dlci;
+		fr->channel  = frm->channel;
+		fr->audio_fd = frm->audio_fd;
 	} else {
 		if (!(fr = get_frame(frm->handle))) {
 			fprintf(stderr, "Not enough connection handles\n");
