@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - Common parser functions
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
  *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
@@ -18,10 +18,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: parser.h,v 1.49 2005/06/18 23:06:26 holtmann Exp $
  */
 
 #ifndef __PARSER_H
@@ -37,6 +35,7 @@ struct frame {
 	uint32_t	data_len;
 	void		*ptr;
 	uint32_t	len;
+	uint16_t	dev_id;
 	uint8_t		in;
 	uint8_t		master;
 	uint16_t	handle;
@@ -46,19 +45,21 @@ struct frame {
 	uint8_t		channel;
 	unsigned long	flags;
 	struct timeval	ts;
+	int		audio_fd;
 };
 
 /* Parser flags */
 #define DUMP_WIDTH	20
 
-#define DUMP_ASCII	0x01
-#define DUMP_HEX	0x02
-#define DUMP_EXT	0x04
-#define DUMP_RAW	0x08
-#define DUMP_BPA	0x10
-#define DUMP_BTSNOOP	0x20
-#define DUMP_VERBOSE	0x40
-#define DUMP_TSTAMP	0x80
+#define DUMP_ASCII	0x0001
+#define DUMP_HEX	0x0002
+#define DUMP_EXT	0x0004
+#define DUMP_RAW	0x0008
+#define DUMP_BPA	0x0010
+#define DUMP_TSTAMP	0x0100
+#define DUMP_VERBOSE	0x0200
+#define DUMP_BTSNOOP	0x1000
+#define DUMP_NOVENDOR	0x2000
 #define DUMP_TYPE_MASK	(DUMP_ASCII | DUMP_HEX | DUMP_EXT)
 
 /* Parser filter */
@@ -93,12 +94,14 @@ struct parser_t {
 	unsigned short defpsm;
 	unsigned short defcompid;
 	int state;
+	int audio_fd;
 };
 
 extern struct parser_t parser;
 
 void init_parser(unsigned long flags, unsigned long filter, 
-		unsigned short defpsm, unsigned short defcompid);
+		unsigned short defpsm, unsigned short defcompid,
+		int audio_fd);
 
 static inline int p_filter(unsigned long f)
 {
