@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - RFCOMM parser
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2001-2002  Wayne Lee <waynelee@qualcomm.com>
  *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
@@ -18,16 +18,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: rfcomm.c,v 1.19 2005/05/15 20:55:15 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
-#include <stdlib.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <sys/types.h>
@@ -117,7 +119,7 @@ static inline void mcc_msc(int level, uint8_t *ptr, int len,
 static inline void mcc_rpn(int level, uint8_t *ptr, int len,
 				long_frame_head *head, mcc_long_frame_head *mcc_head)
 {
-	rpn_msg *rpn = (void*) (ptr - STRUCT_END(rpn_msg, mcc_s_head));
+	rpn_msg *rpn = (void *) (ptr - STRUCT_END(rpn_msg, mcc_s_head));
 
 	printf("RPN %s: ", CR_STR(mcc_head));
 	print_rfcomm_hdr(head, ptr, len);
@@ -137,12 +139,12 @@ static inline void mcc_rpn(int level, uint8_t *ptr, int len,
 		rpn->rpn_val.stop_bit, rpn->rpn_val.parity,
 		rpn->rpn_val.parity_type, rpn->rpn_val.xon_input,
 		rpn->rpn_val.xon_output);
+
 	p_indent(level, 0);
 	printf("rtri %d rtro %d rtci %d rtco %d xon %d xoff %d pm 0x%04x\n",
 		rpn->rpn_val.rtr_input, rpn->rpn_val.rtr_output,
 		rpn->rpn_val.rtc_input, rpn->rpn_val.rtc_output,
-		rpn->rpn_val.xon, rpn->rpn_val.xoff,
-		btohs(*(uint16_t *)&(rpn->rpn_val.pm)));
+		rpn->rpn_val.xon, rpn->rpn_val.xoff, btohs(rpn->rpn_val.pm));
 }
 
 static inline void mcc_rls(int level, uint8_t *ptr, int len,
