@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - SDP parser
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2001-2002  Ricky Yuen <ryuen@qualcomm.com>
  *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
@@ -18,16 +18,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: sdp.c,v 1.37 2005/02/06 14:54:54 holtmann Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
-#include <stdlib.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <sys/types.h>
@@ -100,8 +102,10 @@ static sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_L2CAP,                    "L2CAP"        },
 	{ SDP_UUID_BNEP,                     "BNEP"         }, /* PAN */
 	{ SDP_UUID_HIDP,                     "HIDP"         }, /* HID */
+	{ SDP_UUID_AVCTP,                    "AVCTP"        }, /* AVCTP */
 	{ SDP_UUID_AVDTP,                    "AVDTP"        }, /* AVDTP */
 	{ SDP_UUID_CMTP,                     "CMTP"         }, /* CIP */
+	{ SDP_UUID_UDI_C_PLANE,              "UDI_C-Plane"  }, /* UDI */
 	{ SDP_UUID_SERVICE_DISCOVERY_SERVER, "SDServer"     },
 	{ SDP_UUID_BROWSE_GROUP_DESCRIPTOR,  "BrwsGrpDesc"  },
 	{ SDP_UUID_PUBLIC_BROWSE_GROUP,      "PubBrwsGrp"   },
@@ -116,7 +120,10 @@ static sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_CORDLESS_TELEPHONY,       "CordlessTel"  },
 	{ SDP_UUID_AUDIO_SOURCE,             "AudioSource"  }, /* A2DP */
 	{ SDP_UUID_AUDIO_SINK,               "AudioSink"    }, /* A2DP */
+	{ SDP_UUID_AV_REMOTE_TARGET,         "AVRemTarget"  }, /* AVRCP */
 	{ SDP_UUID_ADVANCED_AUDIO,           "AdvAudio"     }, /* A2DP */
+	{ SDP_UUID_AV_REMOTE,                "AVRemote"     }, /* AVRCP */
+	{ SDP_UUID_VIDEO_CONFERENCING,       "VideoConf"    }, /* VCP */
 	{ SDP_UUID_INTERCOM,                 "Intercom"     },
 	{ SDP_UUID_FAX,                      "Fax"          },
 	{ SDP_UUID_HEADSET_AUDIO_GATEWAY,    "Headset AG"   },
@@ -125,23 +132,41 @@ static sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_PANU,                     "PANU"         }, /* PAN */
 	{ SDP_UUID_NAP,                      "NAP"          }, /* PAN */
 	{ SDP_UUID_GN,                       "GN"           }, /* PAN */
+	{ SDP_UUID_DIRECT_PRINTING,          "DirectPrint"  }, /* BPP */
+	{ SDP_UUID_REFERENCE_PRINTING,       "RefPrint"     }, /* BPP */
 	{ SDP_UUID_IMAGING,                  "Imaging"      }, /* BIP */
 	{ SDP_UUID_IMAGING_RESPONDER,        "ImagingResp"  }, /* BIP */
 	{ SDP_UUID_HANDSFREE,                "Handsfree"    },
 	{ SDP_UUID_HANDSFREE_AUDIO_GATEWAY,  "Handsfree AG" },
+	{ SDP_UUID_DIRECT_PRINTING_REF_OBJS, "RefObjsPrint" }, /* BPP */
+	{ SDP_UUID_REFLECTED_UI,             "ReflectedUI"  }, /* BPP */
+	{ SDP_UUID_BASIC_PRINTING,           "BasicPrint"   }, /* BPP */
+	{ SDP_UUID_PRINTING_STATUS,          "PrintStatus"  }, /* BPP */
 	{ SDP_UUID_HUMAN_INTERFACE_DEVICE,   "HID"          }, /* HID */
-	{ SDP_UUID_HARDCOPY_CABLE_REPLACE,   "HCRP",        }, /* HCRP */
-	{ SDP_UUID_HCR_PRINT,                "HCRPrint",    }, /* HCRP */
-	{ SDP_UUID_HCR_SCAN,                 "HCRScan",     }, /* HCRP */
+	{ SDP_UUID_HARDCOPY_CABLE_REPLACE,   "HCRP"         }, /* HCRP */
+	{ SDP_UUID_HCR_PRINT,                "HCRPrint"     }, /* HCRP */
+	{ SDP_UUID_HCR_SCAN,                 "HCRScan"      }, /* HCRP */
 	{ SDP_UUID_COMMON_ISDN_ACCESS,       "CIP"          }, /* CIP */
+	{ SDP_UUID_VIDEO_CONFERENCING_GW,    "VideoConf GW" }, /* VCP */
+	{ SDP_UUID_UDI_MT,                   "UDI MT"       }, /* UDI */
+	{ SDP_UUID_UDI_TA,                   "UDI TA"       }, /* UDI */
+	{ SDP_UUID_AUDIO_VIDEO,              "AudioVideo"   }, /* VCP */
 	{ SDP_UUID_SIM_ACCESS,               "SAP"          }, /* SAP */
+	{ SDP_UUID_PHONEBOOK_ACCESS_PCE,     "PBAP PCE"     }, /* PBAP */
+	{ SDP_UUID_PHONEBOOK_ACCESS_PSE,     "PBAP PSE"     }, /* PBAP */
 	{ SDP_UUID_PNP_INFORMATION,          "PNPInfo"      },
 	{ SDP_UUID_GENERIC_NETWORKING,       "Networking"   },
 	{ SDP_UUID_GENERIC_FILE_TRANSFER,    "FileTrnsf"    },
 	{ SDP_UUID_GENERIC_AUDIO,            "Audio"        },
 	{ SDP_UUID_GENERIC_TELEPHONY,        "Telephony"    },
+	{ SDP_UUID_UPNP_SERVICE,             "UPNP"         }, /* ESDP */
+	{ SDP_UUID_UPNP_IP_SERVICE,          "UPNP IP"      }, /* ESDP */
+	{ SDP_UUID_ESDP_UPNP_IP_PAN,         "UPNP PAN"     }, /* ESDP */
+	{ SDP_UUID_ESDP_UPNP_IP_LAP,         "UPNP LAP"     }, /* ESDP */
+	{ SDP_UUID_ESDP_UPNP_L2CAP,          "UPNP L2CAP"   }, /* ESDP */
 	{ SDP_UUID_VIDEO_SOURCE,             "VideoSource"  }, /* VDP */
-	{ SDP_UUID_VIDEO_SINK,               "VideoSink"    }  /* VDP */
+	{ SDP_UUID_VIDEO_SINK,               "VideoSink"    }, /* VDP */
+	{ SDP_UUID_VIDEO_DISTRIBUTION,       "VideoDist"    }, /* VDP */
 };
 
 #define SDP_UUID_NAM_LOOKUP_TABLE_SIZE \
@@ -360,7 +385,7 @@ static inline void print_des(uint8_t de_type, int level, int n, struct frame *fr
 
 static inline void print_de(int level, struct frame *frm, int *split, uint16_t *psm, uint8_t *channel)
 {
-	int n;
+	int n = 0;
 	uint8_t de_type = parse_de_hdr(frm, &n);
 
 	switch (de_type) {
@@ -403,7 +428,7 @@ static inline void print_de(int level, struct frame *frm, int *split, uint16_t *
 
 static inline void print_srv_srch_pat(int level, struct frame *frm)
 {
-	int len, n1, n2;
+	int len, n1 = 0, n2 = 0;
 
 	p_indent(level, frm);
 	printf("pat");
@@ -411,7 +436,7 @@ static inline void print_srv_srch_pat(int level, struct frame *frm)
 	if (parse_de_hdr(frm, &n1) == SDP_DE_SEQ) {
 		len = frm->len;
 		while (len - frm->len < n1 && frm->len > 0) {
-			if (parse_de_hdr(frm,&n2) == SDP_DE_UUID) {
+			if (parse_de_hdr(frm, &n2) == SDP_DE_UUID) {
 				print_uuid(n2, frm, NULL, NULL);
 			} else {
 				printf("\nERROR: Unexpected syntax (UUID)\n");
@@ -429,7 +454,7 @@ static inline void print_attr_id_list(int level, struct frame *frm)
 {
 	uint16_t attr_id;
 	uint32_t attr_id_range;
-	int len, n1, n2;
+	int len, n1 = 0, n2 = 0;
 
 	p_indent(level, frm);
 	printf("aid(s)");
@@ -471,7 +496,7 @@ static inline void print_attr_list(int level, struct frame *frm)
 {
 	uint16_t attr_id, psm;
 	uint8_t channel;
-	int len, n1, n2, split;
+	int len, split, n1 = 0, n2 = 0;
 
 	if (parse_de_hdr(frm, &n1) == SDP_DE_SEQ) {
 		len = frm->len;
@@ -509,7 +534,7 @@ static inline void print_attr_list(int level, struct frame *frm)
 
 static inline void print_attr_lists(int level, struct frame *frm)
 {
-	int n, cnt = 0;
+	int n = 0, cnt = 0;
 	int count = frm->len;
 
 	if (parse_de_hdr(frm, &n) == SDP_DE_SEQ) {
@@ -597,16 +622,18 @@ static int add_frame(struct frame *frm, int count)
 	if (fr->data)
 		free(fr->data);
 
-	fr->data = data;
+	fr->data     = data;
 	fr->data_len = len + count;
-	fr->len = fr->data_len;
-	fr->ptr = fr->data;
-	fr->in  = frm->in;
-	fr->ts  = frm->ts;
-	fr->handle  = frm->handle;
-	fr->cid     = frm->cid;
-	fr->num     = frm->num;
-	fr->channel = frm->channel;
+	fr->len      = fr->data_len;
+	fr->ptr      = fr->data;
+	fr->dev_id   = frm->dev_id;
+	fr->in       = frm->in;
+	fr->ts       = frm->ts;
+	fr->handle   = frm->handle;
+	fr->cid      = frm->cid;
+	fr->num      = frm->num;
+	fr->channel  = frm->channel;
+	fr->audio_fd = frm->audio_fd;
 
 	return pos;
 }
