@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - BPA sniffer
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
  *
@@ -17,10 +17,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: bpasniff.c,v 1.5 2005/04/22 13:49:34 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -80,7 +78,7 @@ static int read_revision(int dd, char *revision, int size)
 	}
 
 	if (revision)
-		strncpy(revision, buf + 1, size);
+		strncpy(revision, (char *) (buf + 1), size);
 
 	return 0;
 }
@@ -453,7 +451,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	init_parser(DUMP_EXT | DUMP_VERBOSE, ~0L, 0, DEFAULT_COMPID);
+	init_parser(DUMP_EXT | DUMP_VERBOSE, ~0L, 0, DEFAULT_COMPID, -1);
 
 	process_frames(dev);
 
