@@ -1,6 +1,6 @@
 /*
  *
- *  Bluetooth packet analyzer - HCI sniffer
+ *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
  *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
@@ -18,10 +18,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: hcidump.c,v 1.57 2005/07/03 15:22:32 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,7 +74,8 @@ enum {
 	READ,
 	WRITE,
 	RECEIVE,
-	SEND
+	SEND,
+	AUDIO
 };
 
 /* Default options */
@@ -88,7 +87,8 @@ static int  mode = PARSE;
 static int  permcheck = 1;
 static long flags;
 static long filter;
-static char *dump_file;
+static char *dump_file = NULL;
+static char *audio_file = NULL;
 static in_addr_t dump_addr = INADDR_LOOPBACK;
 static in_port_t dump_port = DEFAULT_PORT;
 
@@ -190,8 +190,12 @@ static void process_frames(int dev, int sock, int fd, unsigned long flags)
 		exit(1);
 	}
 
-	printf("device: hci%d snap_len: %d filter: 0x%lx\n", 
-		dev, snap_len, filter); 
+	if (dev == HCI_DEV_NONE)
+		printf("system: ");
+	else
+		printf("device: hci%d ", dev);
+
+	printf("snap_len: %d filter: 0x%lx\n", snap_len, filter);
 
 	memset(&msg, 0, sizeof(msg));
 
@@ -212,7 +216,10 @@ static void process_frames(int dev, int sock, int fd, unsigned long flags)
 
 		/* Process control message */
 		frm.data_len = len;
+		frm.dev_id = dev;
 		frm.in = 0;
+		frm.audio_fd = parser.audio_fd;
+
 		cmsg = CMSG_FIRSTHDR(&msg);
 		while (cmsg) {
 			switch (cmsg->cmsg_type) {
@@ -351,7 +358,7 @@ static int open_file(char *file, int mode, unsigned long flags)
 	struct btsnoop_hdr hdr;
 	int fd, len, open_flags;
 
-	if (mode == WRITE) {
+	if (mode == WRITE || mode == AUDIO) {
 		if (flags & DUMP_BTSNOOP)
 			open_flags = O_WRONLY | O_CREAT;
 		else
@@ -430,7 +437,7 @@ static int open_socket(int dev, unsigned long flags)
 	struct hci_dev_info di;
 	int sk, dd, opt;
 
-	if (permcheck) {
+	if (permcheck && dev != HCI_DEV_NONE) {
 		dd = hci_open_dev(dev);
 		if (dd < 0) {
 			perror("Can't open device");
@@ -532,7 +539,8 @@ static int wait_connection(in_addr_t addr, in_port_t port)
 {
 	struct sockaddr_in sa;
 	struct hostent *host;
-	int sk, nsk, opt, len;
+	socklen_t len;
+	int sk, nsk, opt;
 
 	sk = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 	if (sk < 0) {
@@ -564,7 +572,8 @@ static int wait_connection(in_addr_t addr, in_port_t port)
 	}
 
 	len = sizeof(sa);
-	if ((nsk = accept(sk, (struct sockaddr *) &sa, &len)) < 0) {
+	nsk = accept(sk, (struct sockaddr *) &sa, &len);
+	if (nsk < 0) {
 		perror("Can't accept new inet socket");
 		close(sk);
 		exit(1);
@@ -635,8 +644,10 @@ static void usage(void)
 	"  -C, --cmtp=psm             PSM for CMTP\n"
 	"  -H, --hcrp=psm             PSM for HCRP\n"
 	"  -O, --obex=channel         Channel for OBEX\n"
+	"  -A, --audio=file           Extract SCO audio data\n"
 	"  -B, --btsnoop              Use BTSnoop file format\n"
 	"  -V, --verbose              Verbose decoding\n"
+	"  -Y, --novendor             No vendor commands or events\n"
 	"  -h, --help                 Give this help list\n"
 	"      --usage                Give a short usage message\n"
 	);
@@ -659,8 +670,11 @@ static struct option main_options[] = {
 	{ "cmtp",		1, 0, 'C' },
 	{ "hcrp",		1, 0, 'H' },
 	{ "obex",		1, 0, 'O' },
+	{ "audio",		1, 0, 'A' },
 	{ "btsnoop",		0, 0, 'B' },
 	{ "verbose",		0, 0, 'V' },
+	{ "novendor",		0, 0, 'Y' },
+	{ "nopermcheck",	0, 0, 'Z' },
 	{ "help",		0, 0, 'h' },
 	{ 0 }
 };
@@ -669,14 +683,17 @@ int main(int argc, char *argv[])
 {
 	struct hostent *host;
 	struct in_addr addr;
-	int opt;
+	int opt, fd = -1;
 
 	printf("HCI sniffer - Bluetooth packet analyzer ver %s\n", VERSION);
 
-	while ((opt=getopt_long(argc, argv, "i:l:p:m:w:r:s:n:taxXRC:H:O:BVZh", main_options, NULL)) != -1) {
+	while ((opt=getopt_long(argc, argv, "i:l:p:m:w:r:s:n:taxXRC:H:O:A:BVYZh", main_options, NULL)) != -1) {
 		switch(opt) {
 		case 'i':
-			device = atoi(optarg + 3);
+			if (strcasecmp(optarg, "none") && strcasecmp(optarg, "system"))
+				device = atoi(optarg + 3);
+			else
+				device = HCI_DEV_NONE;
 			break;
 
 		case 'l': 
@@ -759,6 +776,10 @@ int main(int argc, char *argv[])
 			set_proto(0, 0, atoi(optarg), SDP_UUID_OBEX);
 			break;
 
+		case 'A':
+			audio_file = strdup(optarg);
+			break;
+
 		case 'B':
 			flags |= DUMP_BTSNOOP;
 			break;
@@ -767,6 +788,10 @@ int main(int argc, char *argv[])
 			flags |= DUMP_VERBOSE;
 			break;
 
+		case 'Y':
+			flags |= DUMP_NOVENDOR;
+			break;
+
 		case 'Z':
 			permcheck = 0;
 			break;
@@ -789,14 +814,17 @@ int main(int argc, char *argv[])
 	if (!filter)
 		filter = ~0L;
 
+	if (audio_file)
+		fd = open_file(audio_file, AUDIO, flags);
+
 	switch (mode) {
 	case PARSE:
-		init_parser(flags, filter, defpsm, defcompid);
+		init_parser(flags, filter, defpsm, defcompid, fd);
 		process_frames(device, open_socket(device, flags), -1, flags);
 		break;
 
 	case READ:
-		init_parser(flags, filter, defpsm, defcompid);
+		init_parser(flags, filter, defpsm, defcompid, fd);
 		read_dump(open_file(dump_file, mode, flags));
 		break;
 
@@ -806,7 +834,7 @@ int main(int argc, char *argv[])
 		break;
 
 	case RECEIVE:
-		init_parser(flags, filter, defpsm, defcompid);
+		init_parser(flags, filter, defpsm, defcompid, fd);
 		read_dump(wait_connection(dump_addr, dump_port));
 		break;
 
